/**
  ******************************************************************************
  * @file    pt32x031_led.c
  * @author  Application Team
  * @version V1.1.0
  * @date    15-September-2020  
  * @brief   This file provides firmware functions to manage the following 
  *          functionalities of the  7SEG LED Display peripheral:
  *           + Initialization and Configuration
  *           + 
  *           + Interrupts and flags management
  *
  *  @verbatim
================================================================================
                      ##### How to use this driver #####
================================================================================
    [..]


  *  @endverbatim
  *
  ******************************************************************************
  * @attention
  *
  * 
  *
  * 
  * 
  * 
  *
  *    
  *
  * 
  * 
  * 
  * 
  * 
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "pt32x031_led.h"

/** @addtogroup PT32x031_StdPeriph_Driver
  * @{
  */

/** @defgroup 7SEG LED Display 
  * @brief 7SEG LED Display driver modules
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
uint32_t GAuB_char_tbl[19]=
{
  Char_0,
  Char_1,
  Char_2,
  Char_3,
  Char_4,
  Char_5,
  Char_6,
  Char_7,
  Char_8,
  Char_9,
  Char_A,
  Char_B,
  Char_C,
  Char_D,
  Char_E,
  Char_F,
  Char_H,
  Char_L,
  Char_U
};

/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/** @defgroup 7SEG LED Display_Private_Functions
  * @{
  */

/** @defgroup 7SEG LED Display_Group1 Initialization and Configuration functions
 *  @brief   Initialization and Configuration functions 
 *
@verbatim
 ===============================================================================
          ##### Initialization and Configuration functions #####
 ===============================================================================
    [..] This section provides functions allowing to:

   
@endverbatim
  * @{
  */

/**
  * @brief  Initializes the LED digital tube peripheral according to the specified parameters
  *         in the LED_InitStruct.
  * @note   This function is used to configure the global features of the LED ( 
  *         LED_DisplayNum, LED_DisplayTime, LED_ClockPrescaler, and eLED_TurnTime).             
  * @param  LED_InitStruct: pointer to an LED_InitTypeDef structure that contains 
  *         the configuration information for the specified LED digital tube.
	          @param  LED_DisplayTimeLighting time of LED.
						        This parameter is between 0 and 0xFF.
						@param  LED_TurnTime: Switching time of LED.
					        	This parameter is between 0 and 0xFF.
						@param  LED_DisplayNum: How many nixie tubes turn on the lights in turn.
						        @arg LED_Display_PosX1 
										@arg LED_Display_PosX2
										@arg LED_Display_PosX3
										@arg LED_Display_PosX4
						@param  LED_ClockPrescaler: Pre-crossover factor of LED.
						        This parameter is between 0 and 0xFF.
  * @retval None
  */
void LED_Init(LED_InitTypeDef* LED_InitStruct)
{
  uint32_t tmpreg = 0;

  /* Check the parameters */
  assert_param(IS_LED_DISPLAY_TIME(LED_InitStruct->LED_DisplayTime));
	assert_param(IS_LED_TURN_TIME(LED_InitStruct->LED_TurnTime));
	assert_param(IS_LED_DISPOS(LED_InitStruct->LED_DisplayNum));
	assert_param(IS_LED_CLOCK_DIV(LED_InitStruct->LED_ClockPrescaler));
  tmpreg = LED->CR;
  tmpreg &= (uint32_t)~(uint32_t)LED_CR_COMSEL;
  tmpreg  |= ((uint32_t)(LED_InitStruct->LED_DisplayNum));

  /* Write to LED CR */
  LED->CR = tmpreg;
	/* Set LED Clock Prescaler */
	LED->CPSR = LED_InitStruct->LED_ClockPrescaler;
	/* Set LED Display Time */	
  LED->TR = 	(LED_InitStruct->LED_DisplayTime) | ((LED_InitStruct->LED_TurnTime)<<0x8);
	
	
}

/**
  * @brief  Fills each LED_InitStruct member with its default value.
  * @note   This function is used to initialize the global features of the LED ( 
  *         LED_DisplayNum, LED_DisplayTime, LED_ClockPrescaler, and eLED_TurnTime).
  * @param  LED_InitStruct: pointer to an LED_InitTypeDef structure which will 
  *         be initialized.
            @param  LED_DisplayTimeLighting time of LED.
						        This parameter is between 0 and 0xFF.
						@param  LED_TurnTime: Switching time of LED.
					        	This parameter is between 0 and 0xFF.
						@param  LED_DisplayNum: How many nixie tubes turn on the lights in turn.
						        @arg LED_Display_PosX1 
										@arg LED_Display_PosX2
										@arg LED_Display_PosX3
										@arg LED_Display_PosX4
						@param  LED_ClockPrescaler: Pre-crossover factor of LED.
						        This parameter is between 0 and 0xFF. 
  * @retval None
  */
void LED_StructInit(LED_InitTypeDef* LED_InitStruct)
{

  LED_InitStruct->LED_DisplayNum = LED_Display_PosX4;

  LED_InitStruct->LED_ClockPrescaler =0x00;
	
	LED_InitStruct->LED_DisplayTime = 0x20;
	
	LED_InitStruct->LED_TurnTime= 0x00;

}

/**
  * @brief  Enables or disables the specified LED digital tube peripheral.
  * @param  NewState: new state of the LED digital tube peripheral. 
  * @note   This parameter can be: ENABLE or DISABLE.
             @arg  ENABLE
             @arg  DISABLE
  * @retval None
  */
void LED_Cmd(FunctionalState NewState)
{
  /* Check the parameters */
  assert_param(IS_FUNCTIONAL_STATE(NewState));

  if (NewState != DISABLE)
  {
    LED->CR |= (uint32_t)LED_CR_EN;
  }
  else
  {
    LED->CR &= (uint32_t)~((uint32_t)LED_CR_EN);
  }
}

/**
  * @brief  Configure the LED Clock Prescaler according to LED_ClockPrescaler.
  * @param  LED_ClockPrescaler:set prescaler coefficient.(0~255)
            This parameter is between 0 and 0xFF. 
  * @retval None
  */
void LED_ClockPrescalerConfig(uint32_t LED_ClockPrescaler)
{
  /* Check the parameters */
  assert_param(IS_LED_CLOCK_DIV(LED_ClockPrescaler));

  /* Configure the LED Clock Prescaler according to LED_ClockPrescaler */
  LED->CPSR = LED_ClockPrescaler;

}

/**
  * @brief  Configure the lighting time of LED.
  * @param  LED_DisplayTime: Configure the lighting time of LED.(0~255)
            This parameter is between 0 and 0xFF. 
  * @retval None
  */
void LED_DisplayTimeConfig(uint32_t LED_DisplayTime)
{
  /* Check the parameters */
  assert_param(IS_LED_DISPLAY_TIME(LED_DisplayTime));

  /* Configure the LED Clock Prescaler according to LED_ClockPrescaler */
	LED->TR &= (uint32_t)~((uint32_t)LED_TR_DISPLAYT);
  LED->TR |= LED_DisplayTime;

}

/**
  * @brief  Configure the switching time of LED.
  * @param  LED_TurnTime: Configure the switching time of LED.(0~255)
            This parameter is between 0 and 0xFF. 
  * @retval None
  */
void LED_TurnTimeConfig(uint32_t LED_TurnTime)
{
  /* Check the parameters */
	assert_param(IS_LED_TURN_TIME(LED_TurnTime));

  /* Configure the LED Clock Prescaler according to LED_ClockPrescaler */
	LED->TR &= (uint32_t)~((uint32_t)LED_TR_TURNT);
  LED->TR |= (uint32_t)(((uint32_t)LED_TurnTime)<<0x08);

}

/**
  * @brief  Configure the LED digital tube Pin as Highsink.
  * @param  LED_COMx
  * @param  NewState: new state of the specified pin Highsink.
  *          This parameter can be: ENABLE or DISABLE.
             @arg  ENABLE
             @arg  DISABLE
  * @retval None
  */
void LED_HighSinkConfig(uint32_t LED_COMx, FunctionalState NewState)
{
  /* Check the parameters */
  assert_param(IS_FUNCTIONAL_STATE(NewState));
  assert_param(IS_LED_COM(LED_COMx));

  if (NewState != DISABLE)
  {
    CMSDK_ANAMIX->HSINK_CON |= LED_COMx;
  }
  else
  {
    CMSDK_ANAMIX->HSINK_CON &= (~(uint32_t)LED_COMx);
  }
}

/**
  * @brief  Displays the specified number at the specified position.
  * @param  LED_COMx: where x can be 1,2,3 and 0 to select the LED peripheral.
            @arg  LED_COM0
            @arg  LED_COM1
            @arg  LED_COM2
            @arg  LED_COM3
  * @param  Data_Index: specifies the digital. ((INDEX) <= 0x12)
            This parameter is between 0 and 0x12.
  * @retval None
  */
void LED_UpdataDisplay(uint32_t LED_COMx, uint32_t Data_Index)
{
  /* Check the parameters */
  assert_param(IS_LED_COM(LED_COMx)); 
	assert_param(IS_LED_DATA_INDEX(Data_Index)); 

	switch(LED_COMx)
		{
		case LED_COM0:  LED->DR0 = GAuB_char_tbl[Data_Index];break;  //ʾλ1
		case LED_COM1:  LED->DR1 = GAuB_char_tbl[Data_Index];break;  //ʾλ2
		case LED_COM2:  LED->DR2 = GAuB_char_tbl[Data_Index];break;  //ʾλ3
		case LED_COM3:  LED->DR3 = GAuB_char_tbl[Data_Index];break;  //ʾλ4	
			
		default: break;
		}	
}


/** @defgroup 7SEG LED Display_Group7 Interrupts and flags management functions
 *  @brief   Interrupts and flags management functions.
 *
@verbatim   
 ===============================================================================
            ##### Interrupts and flags management functions #####
 ===============================================================================
    [..] This section provides functions allowing to configure the ADC Interrupts 
         and get the status and clear flags and Interrupts pending bits.
  
    [..] The ADC provide 6 Interrupts sources and 11 Flags which can be divided into 
         3 groups:

  *** Flags for ADC status ***
  ======================================================
    [..]
        (+)Flags :
           (##) ADC_FLAG_ADRDY : This flag is set after the ADC has been enabled (bit ADEN=1)
               and when the ADC reaches a state where it is ready to accept conversion requests


  *** Flags and Interrupts for ADC channel conversion ***
  =====================================================
    [..]
        (+)Flags :
           (##) ADC_FLAG_EOC : This flag is set by hardware at the end of each conversion
                of a channel when a new data result is available in the data register

        (+)Interrupts :
           (##) ADC_IT_EOC : specifies the interrupt source for end of conversion .

    [..] The user should identify which mode will be used in his application to 
         manage the ADC controller events: Polling mode or Interrupt mode.
  
    [..] In the Polling Mode it is advised to use the following functions:
         (+) ADC_GetFlagStatus() : to check if flags events occur.
         (+) ADC_ClearFlag()     : to clear the flags events.
  
    [..] In the Interrupt Mode it is advised to use the following functions:
         (+) ADC_ITConfig()       : to enable or disable the interrupt source.
         (+) ADC_GetITStatus()    : to check if Interrupt occurs.
         (+) ADC_ClearITPendingBit() : to clear the Interrupt pending Bit 
             (corresponding Flag).

@endverbatim
  * @{
  */

/**
  * @brief  Enables or disables the specified LED interrupts.
  * @param  LED_IT: specifies the LED interrupt sources to be enabled or disabled.
  *         This parameter can be one of the following values:
  *         @arg LED_IT_DISPLAYEND: End of conversion interrupt 
  * @param  NewState: new state of the specified LED interrupts.
  *          This parameter can be: ENABLE or DISABLE.
	          @arg  ENABLE
						@arg  DISABLE
  * @retval None
  */
void LED_ITConfig(uint32_t LED_IT, FunctionalState NewState)
{
  /* Check the parameters */
  assert_param(IS_FUNCTIONAL_STATE(NewState));
  assert_param(IS_LED_CONFIG_IT(LED_IT)); 

  if (NewState != DISABLE)
  {
    LED->CR |= LED_IT;
  }
  else
  {
    LED->CR &= (~(uint32_t)LED_IT);
  }
}

/**
  * @brief  Checks whether the specified LED flag is set or not.
  * @param  LED_FLAG: specifies the flag to check. 
  *          This parameter can be one of the following values:
  *          @arg LED_FLAG_DISPLAYEND: End of conversion flag
  *            
  * @retval The new state of LED_GetFlagStatus (SET or RESET).
  */
FlagStatus LED_GetFlagStatus(uint32_t LED_FLAG)
{
  FlagStatus bitstatus = RESET;

  /* Check the parameters */
  assert_param(IS_LED_GET_FLAG(LED_FLAG));
  if ((LED->EIS & LED_FLAG) != (uint32_t)RESET)
  {
    bitstatus = SET;
  }
  else
  {
    bitstatus = RESET;
  }
  return  bitstatus;
}

/**
  * @brief  Clears the LED's pending flags.
  * @param  LED_FLAG: specifies the flag to clear. 
  *         This parameter can be any combination of the following values:
  *         @arg LED_FLAG_DISPLAYEND: End of conversion flag
  * @retval None
  */
void LED_ClearFlag(uint32_t LED_FLAG)
{
  /* Check the parameters */
  assert_param(IS_LED_CLEAR_FLAG(LED_FLAG));

  LED->EIS = (uint32_t)LED_FLAG;
}

/**
  * @brief  Checks whether the specified LED interrupt has occurred or not.
  * @param  LED_IT: specifies the LED interrupt source to check.
  *         This parameter can be one of the following values:
  *         @arg LED_IT_DISPLAYEND: End of conversion interrupt 
  * @retval The new state of LED_IT (SET or RESET).
  */
ITStatus LED_GetITStatus(uint32_t LED_IT)
{
  ITStatus bitstatus = RESET;
  uint32_t enablestatus = 0;

  /* Check the parameters */
  assert_param(IS_LED_GET_IT(LED_IT));

  enablestatus = (uint32_t)(LED->CR & LED_IT); 

  if (((uint32_t)(LED->EIS & LED_IT) != (uint32_t)RESET) && (enablestatus != (uint32_t)RESET))
  {
    bitstatus = SET;
  }
  else
  {
    bitstatus = RESET;
  }
  return  bitstatus;
}

/**
  * @}
  */

/**
  * @}
  */ 

/**
  * @}
  */ 

/**
  * @}
  */ 

/************************(C) COPYRIGHT PENGPAI Microelectronics*****END OF FILE****/
